@@ -18,6 +18,7 @@ Ext.define('SoL.view.Matches.Actions', {
 
     statics: {
         NEW_TURN_ACTION: 'new_turn',
+        FINAL_TURN_ACTION: 'final_turn',
         SHOW_CLOCK_ACTION: 'show_clock',
         PRINT_CARDS_ACTION: 'print_cards',
         PRINT_RESULTS_ACTION: 'print_results',
@@ -43,6 +44,17 @@ Ext.define('SoL.view.Matches.Actions', {
             handler: me.component.newTurn
         }));
 
+        me.finalTurnAction = me.addAction(new Ext.Action({
+            itemId: ids.FINAL_TURN_ACTION,
+            text: _('Final round'),
+            tooltip: _('Create final round.'),
+            iconCls: 'new-turn-icon',
+            disabled: tourney.currentturn != tourney.rankedturn,
+            hidden: me.component.readOnly || !tourney.finals,
+            scope: me.component,
+            handler: me.component.finalTurn
+        }));
+
         me.showClockAction = me.addAction(new Ext.Action({
             itemId: ids.SHOW_CLOCK_ACTION,
             text: _('Show clock'),
@@ -113,7 +125,8 @@ Ext.define('SoL.view.Matches.Actions', {
 
         var tbar = me.component.child('#ttoolbar');
 
-        tbar.add(0, me.newTurnAction, me.showClockAction, ' ',
+        tbar.add(0, me.newTurnAction, me.finalTurnAction,
+                 me.showClockAction, ' ',
                  me.printMatchesAction,
                  me.printCardsAction,
                  me.printResultsAction,
@@ -383,6 +396,59 @@ Ext.define('SoL.view.Matches', {
         });
     },
 
+    finalTurn: function() {
+        var me = this;
+        var ordinalp = me.statics().ordinalp;
+        var tourney = me.module.tourney;
+        var lbar = me.child('toolbar[dock="left"]');
+
+        if(me.focusedCompetitor) {
+            lbar.show();
+            me.getColumnByName('turn').hide();
+            me.getColumnByName('board').show();
+            me.filterOnTurn(tourney.currentturn);
+            me.focusedCompetitor = null;
+        }
+
+        Ext.Ajax.request({
+            url: '/tourney/finalTurn',
+            params: { idtourney: tourney.idtourney },
+            success: function (r) {
+                var res = Ext.decode(r.responseText);
+                if(!res) {
+                    Ext.MessageBox.alert(
+                        _("Comunication error"),
+                        _('Cannot decode JSON object'));
+                } else {
+                    if(res.success) {
+                        var cturn = res.currentturn;
+
+                        tourney.currentturn = cturn;
+                        tourney.rankedturn = res.rankedturn;
+                        tourney.finalturns = res.finalturns;
+                        tourney.prized = res.prized;
+
+                        lbar.insert(cturn-1, Ext.create('Ext.button.Button', {
+                            itemId: 'turn-' + cturn,
+                            text: cturn,
+                            tooltip: Ext.String.format(
+                                _('Show the matches {0} round.'),
+                                ordinalp(cturn)),
+                            turn: cturn,
+                            handler: function(btn) {
+                                me.filterOnTurn(btn.turn);
+                            }
+                        }));
+                        me.filterOnTurn(cturn);
+                        me.updateActions();
+                    } else {
+                        Ext.MessageBox.alert(_('Error'), res.message);
+                    }
+                }
+            }
+        });
+    },
+
     deleteTurn: function(turn) {
         var me = this;
         var tourney = me.module.tourney;
@@ -402,6 +468,7 @@ Ext.define('SoL.view.Matches', {
 
                         tourney.currentturn = res.currentturn;
                         tourney.rankedturn = res.rankedturn;
+                        tourney.finalturns = res.finalturns;
                         tourney.prized = res.prized;
 
                         if(turn > 1) {
@@ -441,8 +508,6 @@ Ext.define('SoL.view.Matches', {
             value: turn
         });
 
-        me.setTitle(Ext.String.format(_('Matches {0} round'),
-                                      ordinalp(turn)));
         me.child('toolbar[dock="left"]').cascade(function(btn) {
             if(btn.turn == turn) {
                 btn.addCls('active-turn');
@@ -491,10 +556,11 @@ Ext.define('SoL.view.Matches', {
                                 _('Cannot decode JSON object'));
                         } else {
                             if(res.success) {
-                                me.module.reloadRanking();
                                 tourney.currentturn = res.currentturn;
                                 tourney.rankedturn = res.rankedturn;
+                                tourney.finalturns = res.finalturns;
                                 tourney.prized = res.prized;
+                                me.module.reloadRanking();
                                 me.updateActions();
                             } else {
                                 Ext.MessageBox.alert(_('Error'), res.message);
@@ -510,15 +576,21 @@ Ext.define('SoL.view.Matches', {
         var me = this;
         var tourney = me.module.tourney;
         var nta = me.findActionById('new_turn');
+        var fta = me.findActionById('final_turn');
+        var clk = me.findActionById('show_clock');
         var lbar = me.child('toolbar[dock="left"]');
 
         nta.setDisabled(!tourney.partecipants ||
-                        (tourney.currentturn > 0
-                         && tourney.currentturn != tourney.rankedturn));
-        nta.setHidden(tourney.prized);
+                        (tourney.currentturn > 0 && tourney.currentturn!=tourney.rankedturn));
+        nta.setHidden(tourney.prized || me.readOnly);
+        fta.setDisabled(!tourney.partecipants || tourney.finalturns ||
+                        (tourney.currentturn > 0 && tourney.currentturn!=tourney.rankedturn));
+        fta.setHidden(tourney.prized || !tourney.finals || me.readOnly);
+        clk.setHidden(tourney.prized || me.readOnly);
         lbar.cascade(function(btn) {
             if(btn.xtype == 'tbseparator' || btn.iconCls == 'icon-cross') {
-                btn.setVisible(!tourney.prized && tourney.currentturn > 0);
+                btn.setVisible((!tourney.prized && tourney.currentturn > 0) ||
+                               tourney.finalturns);
             }
         });
     }
